/* $Id: font.h,v 1.4 1998/10/25 17:57:12 marcus Exp $
******************************************************************************

   KGIcon - font and character handling

   Copyright (C) 1996	Steffen Seeger	[seeger@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _KGI_FONT_H
#define _KGI_FONT_H

#include <kgi/system.h>

/*\
|*| ----------------------------------------------------------------------------
|*|	characters
|*| ----------------------------------------------------------------------------
|*|	ISO 10646 defines symbol codes as 32 bit unsinged int organized into
|*|	groups, planes, rows and cells. By now only the first plane (group=0,
|*|	plane=0) is defined, which maps directly to UNICODE. Thus isochar to
|*|	font position mapping only cares about UNICODE (but can be extended).
\*/

typedef kgiu32 isochar;
typedef kgiu16 unicode;

#define ISO_GROUP(x)	(x >> 24)
#define	ISO_PLANE(x)	((x >> 16) & 0xFF)
#define	ISO_ROW(x)	((x >> 8) & 0xFF)
#define	ISO_CELL(x)	(x & 0xFF)

#define	ISO_GROUP_MASK	0xFF000000
#define	ISO_PLANE_MASK	0x00FF0000
#define	ISO_ROW_MASK	0x0000FF00
#define	ISO_CELL_MASK	0x000000FF

typedef struct { isochar sym; unsigned int pos; }	ggi_cellinfo;

/*\
|*| ----------------------------------------------------------------------------
|*|	kernel fonts
|*| ----------------------------------------------------------------------------
|*|	The kernel fonts have (at maximum) 256 font positions as this can
|*|	(hopefully) be done or emulated on all displays. To support ISO10646 as
|*|	good and fast as possible, we use a (speed and common case) optimized
|*|	mapping not to exhaust memory:
|*|
|*|	Generally cellinfo[ISO_ROW(x)] points to data about row ISO_ROW(x):
|*|
|*|	  * NULL if the font has no positions that map to this row.
|*|	  * if bit number row of map_direct is set, cellinfo[row] is a pointer
|*|	    to an 'kgiu8 map[256]' array, map[ISO_CELL(sym)] giving the font
|*|	    position. Use this if more than 31 font postions map to this row.
|*|	  * else cellinfo[row] points to an array ggi_cellinfo's, sorted in
|*|	    ascending order of cellinfo.sym and the first giving total number
|*|	    of cells and highest cell assigned.
\*/

  struct kgi_fontinfo
  {	kgi_uint	positions;		/* number of positions defined	*/
	kgi_uint	default_pos;		/* fontpos to use as default	*/
	kgi_uint	map_direct[8];
	void	*cellinfo[256];		/* map info for iso -> fontpos	*/
	isochar	inversemap[256];	/* map info for fontpos -> iso	*/
  };

  struct kgi_font
  {	kgi_uint	height, width;		/* distance between char's	*/
	kgi_uint	dx, dy;			/* size of bitmap per char	*/
	kgi_uint	base;			/* base line			*/

	struct kgi_fontinfo *info;	/* iso10646 <-> fontpos mapping	*/
	void	*data;			/* bitmaps			*/
  };

#ifdef NEED_KGI_PROTOTYPES

  #ifdef __KERNEL__
    #define MAX_NR_DEFAULT_FONTS 4
    extern struct kgi_font *kgi_default_font(kgi_modereq *mode);
    extern kgi_uint  kgi_fontpos(struct kgi_font *font, isochar sym);
  #endif
#endif	/* #ifdef NEED_KGI_PROTOTYPES */

/*\
|*| ----------------------------------------------------------------------------
|*|	application fonts
|*| ----------------------------------------------------------------------------
|*| !!!	application fonts will have any number of font postitions and
|*| !!!	'usual' font attributes, for now we use the same as in kernel space. 
\*/

#ifndef __KERNEL__
  extern kgi_uint	kgi_fontpos(struct kgi_font *font, isochar sym);
#endif

#endif	/* ifdef _KGI_FONT_H */
