#include <kgi/kgi.h>
#include <kgi/module.h>
#include <kgi/kgi_commands.h>

#define	DAC_PRECISION 8

#include "IBM/vga_dac.h"
#include "IBM/clut.inc"

static struct kgi_ramdac ramdac = 
{
	"nVidia",			/* Manufacturer		*/
	"Riva",				/* Model		*/
	GM_8BIT | GM_15BIT |		/* Supported modes	*/
	GM_16BIT | GM_32BIT,
	FL_DAC_8BIT,		 	/* Flags		*/
	{ 0xFFFF, 0xFFFF },		/* Maximum resolution	*/
	NULL,				/* Private data hook	*/
	DAC_8BIT_PB,  			/* Device type		*/
	{0, 250000000 },	/* Normal mode		*/
	{0,         0 },	/* 2xclk mode		*/
	{
	{0,  80000000 },	/* KGIGT_TEXT16	*/
	{0,         0 },	/* KGIGT_TEXT32	*/
	{0, 250000000 },	/* KGIGT_1BIT	*/
	{0, 250000000 },	/* KGIGT_4BIT	*/
	{0, 250000000 },	/* KGIGT_8BIT	*/	
	{0, 250000000 },	/* KGIGT_15BIT	*/
	{0, 250000000 },	/* KGIGT_16BIT	*/
	{0, 250000000 },	/* KGIGT_24BIT	*/
	{0, 250000000 }		/* KGIGT_32BIT	*/
	}
};

/* GGI interface */

int kgim_ramdac_init(struct kgi_card *card, struct kgi_display *dpy)
{
	NOTICE("RAMDAC_init.");
	return EOK;
}

void kgim_ramdac_done(struct kgi_display *dpy)
{
	NOTICE("RAMDAC_done.");
}

int kgim_ramdac_check_mode (struct kgi_display *dpy, struct kgi_mode *mode,
			    enum kgi_tm_cmd *cmd)
{
	uint newclock = mode->dclk;
  
	switch (mode->request.graphtype) {

		case KGIGT_8BIT:
		case KGIGT_16BIT:
		case KGIGT_32BIT:
			break;
		default:
			DEBUG2("couldn't handle graphtype %i", 
				   mode->request.graphtype);
			return -E(RAMDAC,MODE_CHECK);
	}

	switch (*cmd) {

		case CMD_PROPOSE:
			mode->lclk.mul = mode->clk.mul = 1;
			mode->lclk.div = mode->clk.div = 1;
			return EOK;

	case CMD_LOWER:

		if ((newclock < ramdac.clock[mode->request.graphtype].min) ||
			(newclock < ramdac.normal.min)) {

			DEBUG2("DCLK too low (%i Hz).", newclock);
			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if (ramdac.clock[mode->request.graphtype].max < newclock) {
			newclock = ramdac.clock[mode->request.graphtype].max;
		}
		mode->dclk = newclock;
		return EOK;

	case CMD_RAISE:
		if ((newclock > ramdac.clock[mode->request.graphtype].max) ||
			(newclock > ramdac.normal.max)) {

			DEBUG2("DCLK too high (%i Hz).", newclock);
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		if (newclock < ramdac.clock[mode->request.graphtype].min) {

			newclock = ramdac.clock[mode->request.graphtype].min;
		}

		if (newclock < ramdac.normal.min) {

			newclock = ramdac.normal.min;
		}

		mode->dclk = newclock;
		return EOK;

     case CMD_CHECK:
		if ((mode->lclk.mul != 1) || (mode->lclk.div != 1) ||
		    (mode->clk.mul != 1) || (mode->clk.div != 1)) {

			return -E(RAMDAC,MODE_CHECK);
		}

		if ((mode->dclk < ramdac.clock[mode->request.graphtype].min) ||
		    (mode->dclk < ramdac.normal.min)) {

			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if ((mode->dclk > ramdac.clock[mode->request.graphtype].max) ||
		    (mode->dclk > ramdac.normal.max)) {
			
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		return EOK;

	default:
		INTERNAL_ERROR;
		return -E(RAMDAC,BOGUS);
	}
}

void kgim_ramdac_set_mode(struct kgi_display *dpy,struct kgi_mode *mode)
{
	switch (mode->request.graphtype) {
		case KGIGT_8BIT:
		case KGIGT_16BIT:
		case KGIGT_32BIT:
			break;
		default:
			INTERNAL_ERROR;
	}

	dpy->set_clut = dac_set_clut;
}

int kgim_ramdac_command(struct kgi_graphic *graph, kgi_uint cmd, kgi_uint arg)
{
	/* This may set 
	 * -  Hardware cursor
	 * -  CLUT (or gamma correction table)
	 */
	kgi_clut clut;

	switch (cmd) { 

		case RAMDAC_GETINFO:
			graph->io_res = &ramdac;
			return EOK;
		
		case RAMDAC_SETINFO:
			if (suser()) {
				memcpy(&ramdac, graph->io_buf, sizeof(ramdac));
				return EOK;
			} 
			else {
				return -EPERM;
			}
		case RAMDAC_SETCLUT:
			clut.size = 256;
			clut.data = (kgi_color *)(graph->io_buf);
			dac_set_clut(graph->dev.dpy, &clut, 0, 256);
			return 0;

		case RAMDAC_GETCLUT:
			clut.size = 256;
			clut.data = (kgi_color *)(graph->io_buf);
			graph->io_res = graph->io_buf;
			dac_get_clut(graph->dev.dpy, &clut, 0, 256);
			return 0;
	}
  	return -E(RAMDAC, NOSUP_ALWAYS_CANT);  /* The ramdac can't do this */
}
